<?php

namespace Drupal\flysystem_dropbox\Flysystem;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\flysystem\Flysystem\Adapter\MissingAdapter;
use Drupal\flysystem\Plugin\FlysystemPluginInterface;
use Drupal\flysystem\Plugin\FlysystemUrlTrait;
use Drupal\flysystem\Plugin\ImageStyleGenerationTrait;
use Drupal\flysystem_dropbox\Services\DropboxTokenService;
use GuzzleHttp\Psr7\Uri;
use Spatie\Dropbox\Client;
use Spatie\FlysystemDropbox\DropboxAdapter;
use Stevenmaguire\OAuth2\Client\Provider\Dropbox as DropboxProvider;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Drupal plugin for the "Dropbox" Flysystem adapter.
 *
 * @Adapter(id = "dropbox")
 */
final class Dropbox implements FlysystemPluginInterface, ContainerFactoryPluginInterface {

  use FlysystemUrlTrait {
    getExternalUrl as getDownloadlUrl;
  }

  use ImageStyleGenerationTrait;

  /**
   * The Dropbox client.
   */
  protected Client $client;

  /**
   * The path prefix inside the Dropbox folder.
   */
  protected string $prefix;

  /**
   * Whether to serve files via Dropbox.
   */
  protected bool $usePublic;

  /**
   * The configuration factory service.
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The Dropbox token service.
   */
  protected DropboxTokenService $dropboxTokenService;

  /**
   * Constructs a Dropbox object.
   *
   * @param non-empty-array<string> $configuration
   *   Plugin configuration array.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory holding resource settings.
   * @param \Drupal\flysystem_dropbox\Services\DropboxTokenService $dropbox_token_service
   *   The Dropbox token service.
   */
  public function __construct(
    array $configuration,
    ConfigFactoryInterface $config_factory,
    DropboxTokenService $dropbox_token_service,
  ) {
    $this->prefix = $configuration['prefix'] ?? '';
    $this->usePublic = !empty($configuration['public']);
    $this->configFactory = $config_factory;
    $this->dropboxTokenService = $dropbox_token_service;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container interface.
   * @param non-empty-array<string> $configuration
   *   Plugin configuration array.
   * @param mixed $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    mixed $plugin_id,
    mixed $plugin_definition,
  ): ContainerFactoryPluginInterface|Dropbox {
    return new self(
      $configuration,
      $container->get('config.factory'),
      $container->get('flysystem_dropbox.services_dropbox_token_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getAdapter(): MissingAdapter|DropboxAdapter {
    try {
      $adapter = new DropboxAdapter($this->getClient(), $this->prefix);
    }

    catch (\Exception $e) {
      $adapter = new MissingAdapter();
    }

    return $adapter;
  }

  /**
   * {@inheritdoc}
   */
  public function getExternalUrl($uri): string|false {
    if ($this->usePublic) {
      return $this->getPublicUrl($uri);
    }

    return $this->getDownloadlUrl($uri);
  }

  /**
   * {@inheritdoc}
   *
   * @return array<int, array<string, array<string, string>|int|string>>
   *   Array.
   */
  public function ensure(mixed $force = FALSE): array {
    try {
      $this->getClient()->getAccountInfo();
    }
    catch (\Exception $e) {
      return [[
        'severity' => RfcLogLevel::ERROR,
        'message' => 'The Dropbox client failed with: %error.',
        'context' => ['%error' => $e->getMessage()],
      ],
      ];
    }

    return [];
  }

  /**
   * Returns the public Dropbox URL.
   *
   * @param string $uri
   *   The file URI.
   *
   * @return string|false
   *   The public URL, or false on failure.
   */
  protected function getPublicUrl(string $uri): string|false {
    $target = $this->getTarget($uri);

    // Quick exit for existing files.
    if ($link = $this->getSharableLink($target)) {
      return $link;
    }

    // Support image style generation.
    if ($this->generateImageStyle($target)) {
      return $this->getSharableLink($target);
    }

    return FALSE;
  }

  /**
   * Returns the Dropbox sharable link.
   *
   * @param string $target
   *   The file target.
   *
   * @return string|false
   *   The sharable link, or false on failure.
   */
  protected function getSharableLink(string $target): string|false {
    try {
      $link = $this->getClient()->createSharedLinkWithSettings('/' . $target);
    }
    catch (\Exception $e) {
    }

    if (isset($link) && $link !== []) {
      $url = $link['url'];
      // Assert that the url is of the expected type.
      assert(is_string($url));
      $uri = (new Uri($url))->withHost('dl.dropboxusercontent.com');

      return (string) Uri::withoutQueryValue($uri, 'dl');
    }

    return FALSE;
  }

  /**
   * Returns the Dropbox client.
   *
   * @return \Spatie\Dropbox\Client
   *   The Dropbox client.
   *
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   *   Identity provider exception.
   */
  protected function getClient(): Client {
    $config = $this->configFactory->get('flysystem_dropbox.settings');
    $client_id = $config->get('flysystem_dropbox.client_id');
    $client_secret = $config->get('flysystem_dropbox.client_secret');
    if (!isset($this->client)) {
      $authClient = new DropboxProvider(
            [
              'clientId' => $client_id,
              'clientSecret' => $client_secret,
            ]
        );

      $tokenProvider = $this->dropboxTokenService;
      $tokenProvider->withAuthClient($authClient);
      $token = $tokenProvider->getAccessToken();

      $this->client = new Client($token->getToken(), $authClient->getHttpClient());
    }

    return $this->client;
  }

}
