<?php

namespace Drupal\flysystem_dropbox\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Stevenmaguire\OAuth2\Client\Provider\Dropbox;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Module settings form.
 */
final class ConfigForm extends ConfigFormBase {

  /**
   * The request object.
   */
  protected Request|RequestStack $request;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs the object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The request object.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    RequestStack $request,
    MessengerInterface $messenger,
  ) {
    parent::__construct($config_factory);
    $this->request = $request;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): ConfigFormBase|ConfigForm {
    return new self(
      $container->get('config.factory'),
      $container->get('request_stack'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'flysystem_dropbox_config_form';
  }

  /**
   * {@inheritdoc}
   *
   * @param array<mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   *
   * @return array<mixed>
   *   The rendered form.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);

    // Default settings.
    $config = $this->config('flysystem_dropbox.settings');

    // New Dropbox client so we can construct the authorization URL.
    $authClient = new Dropbox(
          [
            'clientId' => $config->get('flysystem_dropbox.client_id'),
            'clientSecret' => $config->get('flysystem_dropbox.client_secret'),
          ]
      );

    // Construct the authorization URL.
    $authorization_url = $authClient->getAuthorizationUrl(
          [
            'token_access_type' => 'offline',
            'scope' => 'account_info.read files.metadata.read files.content.read files.content.write',
          ]
      );

    // Help text.
    $form['help'] = [
      '#type' => 'item',
      '#title' => $this->t('Instructions'),
      '#markup' => $this->t('Enter your Dropbox ID and secret below. After saving the form, <a href="@authorization_url" target="_blank">click this link</a> to perform the one time refresh_token fetch with Dropbox. You will need to copy and paste the access code that Dropbox provides on the following screen to the access code field below.', ['@authorization_url' => $authorization_url]),
    ];

    // Client ID field.
    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('App key'),
      '#default_value' => $config->get('flysystem_dropbox.client_id'),
      '#description' => $this->t('Your Dropbox app key.'),
    ];

    // Client Secret field.
    $form['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('App secret'),
      '#default_value' => $config->get('flysystem_dropbox.client_secret'),
      '#description' => $this->t('Your Dropbox app secret.'),
    ];

    // Access code field.
    $form['access_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Access code'),
      '#default_value' => $config->get('flysystem_dropbox.access_code'),
      '#description' => $this->t('The access code you receive after providing the above details and clicking the link above.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @param array<mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    if ($form_state->getValue('client_id') == NULL) {
      $form_state->setErrorByName('client_id', $this->t('Please enter a valid Dropbox app key.'));
    }
    if ($form_state->getValue('client_secret') == NULL) {
      $form_state->setErrorByName('client_secret', $this->t('Please enter a valid Dropbox app secret.'));
    }
  }

  /**
   * {@inheritdoc}
   *
   * @param array<mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state interface.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): mixed {
    $config = $this->config('flysystem_dropbox.settings');
    $config->set('flysystem_dropbox.client_id', $form_state->getValue('client_id'));
    $config->set('flysystem_dropbox.client_secret', $form_state->getValue('client_secret'));
    $config->set('flysystem_dropbox.access_code', $form_state->getValue('access_code'));
    $config->save();

    return parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * @return array<string>
   *   The config names.
   */
  protected function getEditableConfigNames(): array {
    return [
      'flysystem_dropbox.settings',
    ];
  }

}
