<?php

namespace Drupal\flysystem_dropbox\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use League\OAuth2\Client\Token\AccessToken;
use League\OAuth2\Client\Token\AccessTokenInterface;
use Stevenmaguire\OAuth2\Client\Provider\Dropbox;

/**
 * Service for handling Dropbox access tokens.
 */
final class DropboxTokenService {

  use StringTranslationTrait;

  /**
   * The path prefix inside the Dropbox folder.
   */
  protected string $prefix;

  /**
   * The OAuth2 Client.
   */
  protected Dropbox $authClient;

  /**
   * The configuration factory service.
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Logger interface.
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * State system.
   */
  protected StateInterface $state;

  /**
   * Constructs a Dropbox object.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   Config Factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Logger.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state handler service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    StateInterface $state,
  ) {
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $this->state = $state;
  }

  /**
   * Use the Auth Client.
   *
   * @param \Stevenmaguire\OAuth2\Client\Provider\Dropbox $auth_client
   *   The OAuth2 Client.
   */
  public function withAuthClient(Dropbox $auth_client): void {
    $this->authClient = $auth_client;
  }

  /**
   * Either use, get, or retrieve a new short-lived access token.
   *
   * @return \League\OAuth2\Client\Token\AccessTokenInterface
   *   The access token.
   *
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   *   Identity provider exception.
   */
  public function getAccessToken(): AccessTokenInterface {
    $token = $this->getToken();
    if (!$token instanceof AccessToken) {
      $config = $this->configFactory->get('flysystem_dropbox.settings');
      $access_code = $config->get('flysystem_dropbox.access_code');

      // Assert that the access_code is of the expected type.
      assert(is_string($access_code));

      $token = $this->setUp($access_code);
    }
    elseif ($token->hasExpired()) {
      $token = $this->refreshToken();
    }

    return $token;
  }

  /**
   * Use the access code to retrieve and save a refresh token.
   *
   * Also store the first short-lived access token.
   *
   * @param string $access_code
   *   The Dropbox access code.
   *
   * @return \League\OAuth2\Client\Token\AccessTokenInterface
   *   The Dropbox access token.
   *
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   *   Identity provider exception.
   */
  public function setUp(string $access_code): AccessTokenInterface {
    $token = $this->authClient->getAccessToken('authorization_code', ['code' => $access_code]);
    $config = $this->configFactory->getEditable('flysystem_dropbox.settings');
    $config->set('flysystem_dropbox.refresh_token', $token->getRefreshToken())->save();
    $this->storeToken($token);

    return $token;
  }

  /**
   * Use the refresh token to retrieve and store a new short-lived access token.
   *
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   *   Identity provider exception.
   */
  public function refreshToken(): AccessTokenInterface {
    $config = $this->configFactory->get('flysystem_dropbox.settings');
    $refresh_token = $config->get('flysystem_dropbox.refresh_token');
    $token = $this->authClient->getAccessToken('refresh_token', ['refresh_token' => $refresh_token]);
    $this->storeToken($token);
    $message = $this->t('Refreshed Dropbox token: @token', ['@token' => $token->getToken()]);
    $this->loggerFactory->get('flysystem_dropbox')->info($message);

    return $token;
  }

  /**
   * Store the short-lived access token.
   *
   * @param \League\OAuth2\Client\Token\AccessTokenInterface $token
   *   The access token.
   *
   * @return void
   *   Nothing.
   */
  public function storeToken(AccessTokenInterface $token): void {
    $this->state->set('flysystem_dropbox_token', $token);
  }

  /**
   * Get the short-lived access token.
   *
   * @return \League\OAuth2\Client\Token\AccessToken
   *   The stored token.
   */
  public function getToken(): AccessToken {
    $token = $this->state->get('flysystem_dropbox_token');

    // Assert that the token is of the expected type.
    assert($token instanceof AccessToken);

    return $token;
  }

}
